﻿var scada = scada || {};

scada.notif = {
    // Current data refresh rate by default
    DEF_DATA_REFR_RATE: 1000,
    // Archive data refresh rate by default
    DEF_ARC_REFR_RATE: 10000,

    // Events data age after full update
    _fullDataAge: 0,
    // Events data age after partial update
    _partialDataAge: 0,
    // String representation of the date of the last processed event table
    _lastEvDate: "",
    // Number of the last processed event
    _lastEvNum: 0,
    // Key of the last received notification
    _lastNotifKey: "",
    // Timeout ID of the full events updating timer
    _fullUpdateTimeoutID: null,
    // Timeout ID of the partial events updating timer
    _partUpdateTimeoutID: null,
    // The view hub object
    _viewHub: null,//scada.viewHubLocator.getViewHub(),
    // Ajax queue used for request sequencing. Can be null
    _ajaxQueue: scada.ajaxQueueLocator.getAjaxQueue(),

    // The notification panel
    notifPanel: scada.notifPanelLocator.getNotifPanel(),

    // Start cyclic updating of all displayed notifications
    _startFullUpdating: function () {
        var thisObj = this;
        this._fullUpdate(function () {
            thisObj._fullUpdateTimeoutID = setTimeout(thisObj._startFullUpdating.bind(thisObj),
                scada.env.arcRefrRate || thisObj.DEF_ARC_REFR_RATE);
        });
    },

    // Start cyclic updating of newly added notifications
    _startPartialUpdating: function () {
        var thisObj = this;
        this._partialUpdate(function () {
            thisObj._partUpdateTimeoutID = setTimeout(thisObj._startPartialUpdating.bind(thisObj),
                scada.env.dataRefrRate || thisObj.DEF_DATA_REFR_RATE);
        });
    },

    // Request and display all notifications
    _fullUpdate: function (callback) {
        var thisObj = this;
        this._requestNotifications("", 0, this._fullDataAge, function (success, notifDTO) {
            if (success) {
                var notifications = thisObj._createNotifications(notifDTO);

                if (notifications.length > 0) {
                    thisObj.notifPanel.replaceNotifications(notifications);
                    var lastNotifKey = notifications[notifications.length - 1].key;

                    if (thisObj._lastNotifKey !== lastNotifKey) {
                        thisObj._lastNotifKey = lastNotifKey;
                        thisObj.notifPanel.show(true, true);
                    }
                } else if (thisObj._fullDataAge !== notifDTO.DataAge) {
                    thisObj.notifPanel.clearNotifications();
                }

                thisObj._fullDataAge = notifDTO.DataAge;
                thisObj._partialDataAge = notifDTO.DataAge;
                thisObj._lastEvDate = notifDTO.LastEvDate;
                thisObj._lastEvNum = notifDTO.LastEvNum;
            }

            callback();
        });
    },

    // Request and display new notifications
    _partialUpdate: function (callback) {
        var thisObj = this;
        this._requestNotifications(this._lastEvDate, this._lastEvNum + 1, this._partialDataAge,
            function (success, notifDTO) {
                if (success) {
                    var notifications = thisObj._createNotifications(notifDTO);

                    if (notifications.length > 0) {
                        thisObj.notifPanel.appendNotifications(notifications);
                        thisObj.notifPanel.show(true, true);
                    }

                    thisObj._partialDataAge = notifDTO.DataAge;
                    thisObj._lastEvDate = notifDTO.LastEvDate;
                    thisObj._lastEvNum = notifDTO.LastEvNum;
                }

                callback();
            });
    },

    // Perform a request to get notifications.
    // callback is a function (success, notifDTO)
    _requestNotifications: function (startEvDate, startEvNum, dataAge, callback) {
        var thisObj = this;
        var ajaxObj = this.ajaxQueue ? this._ajaxQueue : $;
        var operation = "NotifSvc.svc/GetNotifications";

        ajaxObj
            .ajax({
                url: scada.env.rootPath + "plugins/Notification/" + operation +
                    "?startEvDate=" + startEvDate + "&startEvNum=" + startEvNum + "&dataAge=" + dataAge,
                method: "GET",
                dataType: "json",
                cache: false
            })
            .done(function (data, textStatus, jqXHR) {
                try {
                    var parsedData = $.parseJSON(data.d);
                    if (parsedData.Success) {
                        if (thisObj._validateNotifDTO(parsedData)) {
                            scada.utils.logSuccessfulRequest(operation);
                            callback(true, parsedData);
                        } else {
                            scada.utils.logProcessingError(operation, "Invalid received data.");
                            callback(false, null);
                       }
                    } else {
                        scada.utils.logServiceError(operation, parsedData.ErrorMessage);
                        callback(false, null);
                    }
                }
                catch (ex) {
                    scada.utils.logProcessingError(operation, ex.message);
                    if (typeof callback === "function") {
                        callback(false, null);
                    }
                }
            })
            .fail(function (jqXHR, textStatus, errorThrown) {
                scada.utils.logFailedRequest(operation, jqXHR);
                if (typeof callback === "function") {
                    callback(false, null);
                }
            });
    },

    // Validate the received object
    _validateNotifDTO: function (notifDTO) {
        return Array.isArray(notifDTO.Data) && typeof notifDTO.LastEvDate === "string" &&
            Number.isInteger(notifDTO.LastEvNum) && Number.isInteger(notifDTO.DataAge);
    },

    // Create notification array based on the received data
    _createNotifications: function (notifDTO) {
        var notifCnt = notifDTO.Data.length;
        var notifications = new Array(notifCnt);

        for (var i = 0; i < notifCnt; i++) {
            var srcNotif = notifDTO.Data[i];
            notifications[i] = new scada.Notification(srcNotif.Key, srcNotif.DateTime,
                srcNotif.MessageHtml, srcNotif.NotifType);
        }

        return notifications;
    },

    // Create controls for debugging the plugin
    enableDebugMode: function () {
        var controls = $("<div>" +
            "<button type='button'>Show</button>" +
            "<button type='button'>Hide</button>" +
            "<button type='button'>Add</button>" +
            "<button type='button'>Remove</button></div>");

        controls.css({
            "top": 0,
            "left": 250,
            "position": "fixed"
        });

        var thisObj = this;
        var buttons = controls.find("button");
        var notifID = 0;

        buttons.eq(0).click(function () {
            thisObj.notifPanel.show(true);
        });

        buttons.eq(1).click(function () {
            thisObj.notifPanel.hide();
        });

        buttons.eq(2).click(function () {
            var today = new Date();
            notifID = thisObj.notifPanel.addNotification(
                new scada.Notification(
                    today.valueOf().toString(),
                    today.toLocaleString(),
                    "Test notif, test notif, test notif, test notif, test notif, test notif, test notif, test notif",
                    notifID % 3));
        });

        buttons.eq(3).click(function () {
            thisObj.notifPanel.removeNotification(notifID);
        });

        controls.appendTo("body");
    },

    // Start cyclic updating of notifications
    startUpdating: function () {
        this._startFullUpdating();
        this.partUpdateTimeoutID = setTimeout(this._startPartialUpdating.bind(this),
            scada.env.dataRefrRate || scada.notif.DEF_DATA_REFR_RATE);
    },

    // Show event tip dialog
    showEventTip: function (date, evNum) {
        var popup = scada.popupLocator.getPopup();
        if (popup) {
            popup.showModal(scada.env.rootPath + "plugins/Notification/EventTip.aspx?" +
                scada.utils.dateToQueryString(date) + "&evNum=" + evNum,
                new scada.ModalOptions([scada.ModalButtons.CLOSE]));
        }
    },

    // Show event acknowledgement dialog
    showEventAck: function (date, evNum, notifID) {
        if (scada.dialogs) {
            scada.dialogs.showEventAck(date, evNum, 0, function (dialogResult) {
                if (dialogResult) {
                    // remove confirmed notification
                    this.notifPanel.removeNotification(notifID);

                    // hide empty panel
                    if (this.notifPanel.isEmpty()) {
                        this.notifPanel.hide();
                    }
                }
            });
        } else {
            console.warn("Dialogs object is undefined");
        }
    }
};

$(document).ready(function () {
    //scada.notif.enableDebugMode();
    scada.notif.notifPanel.showBell();
    scada.notif.startUpdating();

    scada.notif.notifPanel.panelElem.on("click", ".notif .tip", function () {
        var tipLink = $(this);
        var date = new Date(tipLink.data("year"), tipLink.data("month") - 1, tipLink.data("day"));
        scada.notif.showEventTip(date, tipLink.data("evnum"));
        return false;
    });

    scada.notif.notifPanel.panelElem.on("click", ".notif .ack", function () {
        var ackLink = $(this);
        var date = new Date(ackLink.data("year"), ackLink.data("month") - 1, ackLink.data("day"));
        var notif = ackLink.closest(".notif").data("notif");
        scada.notif.showEventAck(date, ackLink.data("evnum"), notif.id);
        return false;
    });
});
